// src/types.ts
var defaults = {};
var defineNode = (node) => ({
  ...defaults,
  ...node
});

// src/primitive.ts
var string = () => {
  return defineNode({
    typeName: "string"
  });
};
var number = () => {
  return defineNode({
    typeName: "number"
  });
};
var boolean = () => {
  return defineNode({
    typeName: "boolean"
  });
};
var nullType = () => {
  return defineNode({
    typeName: "null"
  });
};
var undefinedType = () => {
  return defineNode({
    typeName: "undefined"
  });
};
var voidType = () => {
  return defineNode({
    typeName: "void"
  });
};
var literal = (literal2) => {
  return defineNode({
    typeName: "literal",
    type: literal2
  });
};
var unknown = () => {
  return defineNode({
    typeName: "unknown"
  });
};
var any = () => {
  return defineNode({
    typeName: "any"
  });
};
var never = () => {
  return defineNode({
    typeName: "never"
  });
};
var regexp = () => {
  return defineNode({
    typeName: "regexp"
  });
};
var bigint = () => {
  return defineNode({
    typeName: "bigint"
  });
};
var date = () => {
  return defineNode({
    typeName: "date"
  });
};
var file = () => {
  return defineNode({
    typeName: "file"
  });
};
var blob = () => {
  return defineNode({
    typeName: "blob"
  });
};
var symbol = (name) => {
  return defineNode({
    typeName: "symbol",
    name
  });
};

// src/complex.ts
var object = (shape) => {
  return defineNode({
    typeName: "object",
    shape
  });
};
var array = (type) => {
  return defineNode({
    typeName: "array",
    type
  });
};
var iterator = (type) => {
  return defineNode({
    typeName: "iterator",
    type
  });
};
var set = (type) => {
  return defineNode({
    typeName: "set",
    type
  });
};
var record = (key, value) => {
  return defineNode({
    typeName: "record",
    key,
    value
  });
};
var map = (key, value) => {
  return defineNode({
    typeName: "map",
    key,
    value
  });
};
var func = (options = {}) => {
  const arguments_ = options.arguments ?? [];
  const return_ = options.return ?? voidType();
  return defineNode({
    typeName: "function",
    arguments: arguments_,
    return: return_
  });
};
var union = (nodes) => {
  return defineNode({
    typeName: "union",
    types: nodes
  });
};
var tuple = (nodes) => {
  return defineNode({
    typeName: "tuple",
    types: nodes
  });
};
var intersection = (nodes) => {
  return defineNode({
    typeName: "intersection",
    types: nodes
  });
};
var formData = (shape) => {
  return defineNode({
    typeName: "formData",
    ...shape && { shape }
    // shape: shape ?? ({} as TShape),
  });
};
var custom = (name, arguments_) => {
  return defineNode({
    typeName: "custom",
    name,
    ...arguments_ && { arguments: arguments_ }
  });
};

// src/circular.ts
var definitionReference = (identifier, text) => {
  return defineNode({
    typeName: "definitionReference",
    identifier,
    text
  });
};
var named = (node, name) => {
  return defineNode({
    typeName: "named",
    node,
    name
  });
};
var circular = () => {
  return defineNode({
    typeName: "circular"
  });
};

export {
  defineNode,
  string,
  number,
  boolean,
  nullType,
  undefinedType,
  voidType,
  literal,
  unknown,
  any,
  never,
  regexp,
  bigint,
  date,
  file,
  blob,
  symbol,
  object,
  array,
  iterator,
  set,
  record,
  map,
  func,
  union,
  tuple,
  intersection,
  formData,
  custom,
  definitionReference,
  named,
  circular
};
