import { AnyNode } from './types.js';
interface BaseDifference<TDifferenceType extends string> {
    type: TDifferenceType;
    path: (string | number)[];
}
interface AddObjectPropertyDifference extends BaseDifference<'add-object-property'> {
    key: string | number;
}
interface RemoveObjectPropertyDifference extends BaseDifference<'remove-object-property'> {
    key: string | number;
}
interface AddTupleElementDifference extends BaseDifference<'add-tuple-element'> {
    index: number;
}
interface RemoveTupleElementDifference extends BaseDifference<'remove-tuple-element'> {
    index: number;
}
interface AddFunctionArgumentDifference extends BaseDifference<'add-function-argument'> {
    index: number;
}
interface RemoveFunctionArgumentDifference extends BaseDifference<'remove-function-argument'> {
    index: number;
}
interface AddUnionMemberDifference extends BaseDifference<'add-union-member'> {
    member: AnyNode;
}
interface RemoveUnionMemberDifference extends BaseDifference<'remove-union-member'> {
    member: AnyNode;
}
interface AddIntersectionMemberDifference extends BaseDifference<'add-intersection-member'> {
    member: AnyNode;
}
interface RemoveIntersectionMemberDifference extends BaseDifference<'remove-intersection-member'> {
    member: AnyNode;
}
interface ChangeTypeDifference extends BaseDifference<'change-type'> {
    expectedType: AnyNode['typeName'];
    actualType: AnyNode['typeName'];
}
interface ChangeCustomTypeDifference extends BaseDifference<'change-custom-type'> {
    expectedCustomType: string;
    actualCustomType: string;
}
type AddCustomArgumentDifference = BaseDifference<'add-custom-argument'>;
type RemoveCustomArgumentDifference = BaseDifference<'remove-custom-argument'>;
type Difference = ChangeTypeDifference | AddObjectPropertyDifference | RemoveObjectPropertyDifference | AddTupleElementDifference | RemoveTupleElementDifference | ChangeCustomTypeDifference | AddCustomArgumentDifference | RemoveCustomArgumentDifference | AddUnionMemberDifference | RemoveUnionMemberDifference | AddIntersectionMemberDifference | RemoveIntersectionMemberDifference | AddFunctionArgumentDifference | RemoveFunctionArgumentDifference;
/**
 * Hashes an object into a stable string.
 * @see https://github.com/TanStack/query/blob/845751180decd86b2feab669e487a2887e12f8b5/packages/query-core/src/utils.ts#L269C1-L269C1
 */
export declare function hashObject(object: object): string;
/**
 *  Returns a list of differences between two nodes as a list of operations mapping the expected node to the actual node.
 *  Nodes cannot have circular references.
 */
export declare function diffNodes(actualNode: AnyNode, expectedNode: AnyNode, path?: (string | number)[]): Difference[];
export {};
//# sourceMappingURL=diff.d.ts.map