"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  UnsupportedError: () => UnsupportedError,
  any: () => any,
  array: () => array,
  bigint: () => bigint,
  blob: () => blob,
  boolean: () => boolean,
  circular: () => circular,
  custom: () => custom,
  date: () => date,
  defineNode: () => defineNode,
  definitionReference: () => definitionReference,
  file: () => file,
  formData: () => formData,
  func: () => func,
  intersection: () => intersection,
  isNodeOptional: () => isNodeOptional,
  iterator: () => iterator,
  literal: () => literal,
  map: () => map,
  mapMultipleInnerNode: () => mapMultipleInnerNode,
  mapObjectNode: () => mapObjectNode,
  named: () => named,
  never: () => never,
  nullType: () => nullType,
  nullable: () => nullable,
  number: () => number,
  object: () => object,
  optional: () => optional,
  printNode: () => printNode,
  record: () => record,
  regexp: () => regexp,
  serializePrimitive: () => serializePrimitive,
  set: () => set,
  string: () => string,
  symbol: () => symbol,
  tuple: () => tuple,
  undefinedType: () => undefinedType,
  union: () => union,
  unionIfMultiple: () => unionIfMultiple,
  unknown: () => unknown,
  voidType: () => voidType
});
module.exports = __toCommonJS(src_exports);

// src/types.ts
var defaults = {};
var defineNode = (node) => ({
  ...defaults,
  ...node
});

// src/primitive.ts
var string = () => {
  return defineNode({
    typeName: "string"
  });
};
var number = () => {
  return defineNode({
    typeName: "number"
  });
};
var boolean = () => {
  return defineNode({
    typeName: "boolean"
  });
};
var nullType = () => {
  return defineNode({
    typeName: "null"
  });
};
var undefinedType = () => {
  return defineNode({
    typeName: "undefined"
  });
};
var voidType = () => {
  return defineNode({
    typeName: "void"
  });
};
var literal = (literal2) => {
  return defineNode({
    typeName: "literal",
    type: literal2
  });
};
var unknown = () => {
  return defineNode({
    typeName: "unknown"
  });
};
var any = () => {
  return defineNode({
    typeName: "any"
  });
};
var never = () => {
  return defineNode({
    typeName: "never"
  });
};
var regexp = () => {
  return defineNode({
    typeName: "regexp"
  });
};
var bigint = () => {
  return defineNode({
    typeName: "bigint"
  });
};
var date = () => {
  return defineNode({
    typeName: "date"
  });
};
var file = () => {
  return defineNode({
    typeName: "file"
  });
};
var blob = () => {
  return defineNode({
    typeName: "blob"
  });
};
var symbol = (name) => {
  return defineNode({
    typeName: "symbol",
    name
  });
};

// src/complex.ts
var object = (shape) => {
  return defineNode({
    typeName: "object",
    shape
  });
};
var array = (type) => {
  return defineNode({
    typeName: "array",
    type
  });
};
var iterator = (type) => {
  return defineNode({
    typeName: "iterator",
    type
  });
};
var set = (type) => {
  return defineNode({
    typeName: "set",
    type
  });
};
var record = (key, value) => {
  return defineNode({
    typeName: "record",
    key,
    value
  });
};
var map = (key, value) => {
  return defineNode({
    typeName: "map",
    key,
    value
  });
};
var func = (options = {}) => {
  const arguments_ = options.arguments ?? [];
  const return_ = options.return ?? voidType();
  return defineNode({
    typeName: "function",
    arguments: arguments_,
    return: return_
  });
};
var union = (nodes) => {
  return defineNode({
    typeName: "union",
    types: nodes
  });
};
var tuple = (nodes) => {
  return defineNode({
    typeName: "tuple",
    types: nodes
  });
};
var intersection = (nodes) => {
  return defineNode({
    typeName: "intersection",
    types: nodes
  });
};
var formData = (shape) => {
  return defineNode({
    typeName: "formData",
    ...shape && { shape }
    // shape: shape ?? ({} as TShape),
  });
};
var custom = (name, arguments_) => {
  return defineNode({
    typeName: "custom",
    name,
    ...arguments_ && { arguments: arguments_ }
  });
};

// src/composed.ts
var nullable = (type) => {
  return union([type, nullType()]);
};
var optional = (type) => {
  return union([type, undefinedType()]);
};

// src/circular.ts
var definitionReference = (identifier, text) => {
  return defineNode({
    typeName: "definitionReference",
    identifier,
    text
  });
};
var named = (node, name) => {
  return defineNode({
    typeName: "named",
    node,
    name
  });
};
var circular = () => {
  return defineNode({
    typeName: "circular"
  });
};

// src/utils.ts
var mapMultipleInnerNode = (node, mapper) => node.types.map((type) => mapper(type));
var mapObjectNode = (node, mapper) => Object.entries(node.shape).map(
  ([key, value]) => [key, mapper(value)]
);
var serializePrimitive = (value) => {
  switch (typeof value) {
    case "string": {
      return JSON.stringify(value);
    }
    case "number": {
      return value.toString();
    }
    case "boolean": {
      return value === true ? "true" : "false";
    }
    case "bigint": {
      return `${value.toString()}n`;
    }
  }
};
var isNodeOptional = (node) => {
  if (node.typeName === "undefined") {
    return true;
  }
  if (node.typeName === "union") {
    return node.types.some((type) => isNodeOptional(type));
  }
  return false;
};
var unionIfMultiple = (nodes) => {
  if (nodes.length === 1) {
    return nodes[0];
  }
  return union(nodes);
};
var UnsupportedError = class extends Error {
  constructor(integration, feature) {
    super(`the onetyped ${integration} integration does not support ${feature}`);
    this.name = "UnsupportedError";
    this.integration = integration;
    this.feature = feature;
    Object.setPrototypeOf(this, UnsupportedError.prototype);
  }
};

// src/print-node.ts
function printNode(node, options) {
  const printedNode = printNodeInternal(node, options);
  return printedNode;
}
function printNodeInternal(node, options) {
  const { indent = 2, depth = 0 } = options ?? {};
  const baseIndentString = " ".repeat(indent * depth);
  const nestedIndentString = " ".repeat(indent * (depth + 1));
  switch (node.typeName) {
    case "any":
    case "boolean":
    case "never":
    case "null":
    case "number":
    case "string":
    case "undefined":
    case "unknown":
    case "void": {
      return node.typeName;
    }
    case "bigint": {
      return "BigInt";
    }
    case "blob": {
      return "Blob";
    }
    case "date": {
      return "Date";
    }
    case "file": {
      return "File";
    }
    case "regexp": {
      return "RegExp";
    }
    case "array": {
      return `(${printNodeInternal(node.type, options)})[]`;
    }
    case "intersection": {
      return "(" + node.types.map((t) => printNodeInternal(t, options)).join(" & ") + ")";
    }
    case "literal": {
      return JSON.stringify(node.type);
    }
    case "map": {
      return `Map<${printNodeInternal(node.key, options)}, ${printNodeInternal(
        node.value,
        options
      )}>`;
    }
    case "record": {
      return `Record<${printNodeInternal(
        node.key,
        options
      )}, ${printNodeInternal(node.value, options)}>`;
    }
    case "set": {
      return `Set<${printNodeInternal(node.type, options)}>`;
    }
    case "tuple": {
      return `[${node.types.map((t) => printNodeInternal(t, options)).join(", ")}]`;
    }
    case "union": {
      return "(" + node.types.map((t) => printNodeInternal(t, options)).join(" | ") + ")";
    }
    case "iterator": {
      return `Iterator<${printNodeInternal(node.type, options)}>`;
    }
    case "symbol": {
      return `Symbol(${node.name})`;
    }
    case "formData": {
      if (!node.shape) {
        return "FormData";
      }
      const printedShape = Object.entries(node.shape).map(
        ([name, n]) => {
          return `${nestedIndentString}${name}: ${printNodeInternal(n, {
            ...options,
            depth: depth + 1
          })}`;
        }
      );
      return `FormData {
${printedShape.join(";\n")};
${baseIndentString}}`;
    }
    case "object": {
      const printedShape = Object.entries(node.shape).map(
        ([name, n]) => {
          return `${nestedIndentString}${name}: ${printNodeInternal(n, {
            ...options,
            depth: depth + 1
          })}`;
        }
      );
      return `{
${printedShape.join(";\n")};
${baseIndentString}}`;
    }
    case "function": {
      const printedArguments = node.arguments.map(
        (a, index) => `args_${index}: ${printNodeInternal(a, options)}`
      ).join(", ");
      return `(${printedArguments}) => ${printNodeInternal(
        node.return,
        options
      )};`;
    }
    case "custom": {
      if (!node.arguments) {
        return node.name;
      }
      return `${node.name}<${node.arguments.map((a) => printNodeInternal(a, options)).join(", ")}>`;
    }
    case "named": {
      return node.name;
    }
    case "definitionReference": {
      return `DefinitionReference<${node.text}>`;
    }
    case "circular": {
      return "[Circular]";
    }
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  UnsupportedError,
  any,
  array,
  bigint,
  blob,
  boolean,
  circular,
  custom,
  date,
  defineNode,
  definitionReference,
  file,
  formData,
  func,
  intersection,
  isNodeOptional,
  iterator,
  literal,
  map,
  mapMultipleInnerNode,
  mapObjectNode,
  named,
  never,
  nullType,
  nullable,
  number,
  object,
  optional,
  printNode,
  record,
  regexp,
  serializePrimitive,
  set,
  string,
  symbol,
  tuple,
  undefinedType,
  union,
  unionIfMultiple,
  unknown,
  voidType
});
