"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/transform.ts
var transform_exports = {};
__export(transform_exports, {
  visitEachChild: () => visitEachChild,
  visitNode: () => visitNode
});
module.exports = __toCommonJS(transform_exports);

// src/types.ts
var defaults = {};
var defineNode = (node) => ({
  ...defaults,
  ...node
});

// src/circular.ts
var named = (node, name) => {
  return defineNode({
    typeName: "named",
    node,
    name
  });
};

// src/primitive.ts
var voidType = () => {
  return defineNode({
    typeName: "void"
  });
};

// src/complex.ts
var object = (shape) => {
  return defineNode({
    typeName: "object",
    shape
  });
};
var array = (type) => {
  return defineNode({
    typeName: "array",
    type
  });
};
var iterator = (type) => {
  return defineNode({
    typeName: "iterator",
    type
  });
};
var set = (type) => {
  return defineNode({
    typeName: "set",
    type
  });
};
var record = (key, value) => {
  return defineNode({
    typeName: "record",
    key,
    value
  });
};
var map = (key, value) => {
  return defineNode({
    typeName: "map",
    key,
    value
  });
};
var func = (options = {}) => {
  const arguments_ = options.arguments ?? [];
  const return_ = options.return ?? voidType();
  return defineNode({
    typeName: "function",
    arguments: arguments_,
    return: return_
  });
};
var union = (nodes) => {
  return defineNode({
    typeName: "union",
    types: nodes
  });
};
var tuple = (nodes) => {
  return defineNode({
    typeName: "tuple",
    types: nodes
  });
};
var intersection = (nodes) => {
  return defineNode({
    typeName: "intersection",
    types: nodes
  });
};
var custom = (name, arguments_) => {
  return defineNode({
    typeName: "custom",
    name,
    ...arguments_ && { arguments: arguments_ }
  });
};

// src/transform.ts
function visitNode(node, visitor) {
  return visitor(node);
}
function visitEachChild(node, visitor) {
  switch (node.typeName) {
    case "object": {
      return object(
        Object.fromEntries(
          Object.entries(node.shape).map(([key, value]) => [
            key,
            visitNode(value, visitor)
          ])
        )
      );
    }
    case "array": {
      return array(visitNode(node.type, visitor));
    }
    case "tuple": {
      return tuple(node.types.map((node2) => visitNode(node2, visitor)));
    }
    case "record": {
      return record(
        visitNode(node.key, visitor),
        visitNode(node.value, visitor)
      );
    }
    case "set": {
      return set(visitNode(node.type, visitor));
    }
    case "map": {
      return map(visitNode(node.key, visitor), visitNode(node.value, visitor));
    }
    case "function": {
      return func({
        arguments: node.arguments.map(
          (node2) => visitNode(node2, visitor)
        ),
        return: visitNode(node.return, visitor)
      });
    }
    case "intersection": {
      return intersection(
        node.types.map((node2) => visitNode(node2, visitor))
      );
    }
    case "union": {
      return union(node.types.map((node2) => visitNode(node2, visitor)));
    }
    case "named": {
      return named(visitNode(node.node, visitor), node.name);
    }
    case "iterator": {
      return iterator(visitNode(node.type, visitor));
    }
    case "custom": {
      return custom(
        node.name,
        node.arguments?.map((node2) => visitNode(node2, visitor))
      );
    }
    default: {
      return node;
    }
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  visitEachChild,
  visitNode
});
