import {
  array,
  custom,
  func,
  intersection,
  iterator,
  map,
  named,
  object,
  record,
  set,
  tuple,
  union
} from "./chunk-JWRHR5EW.js";

// src/transform.ts
function visitNode(node, visitor) {
  return visitor(node);
}
function visitEachChild(node, visitor) {
  switch (node.typeName) {
    case "object": {
      return object(
        Object.fromEntries(
          Object.entries(node.shape).map(([key, value]) => [
            key,
            visitNode(value, visitor)
          ])
        )
      );
    }
    case "array": {
      return array(visitNode(node.type, visitor));
    }
    case "tuple": {
      return tuple(node.types.map((node2) => visitNode(node2, visitor)));
    }
    case "record": {
      return record(
        visitNode(node.key, visitor),
        visitNode(node.value, visitor)
      );
    }
    case "set": {
      return set(visitNode(node.type, visitor));
    }
    case "map": {
      return map(visitNode(node.key, visitor), visitNode(node.value, visitor));
    }
    case "function": {
      return func({
        arguments: node.arguments.map(
          (node2) => visitNode(node2, visitor)
        ),
        return: visitNode(node.return, visitor)
      });
    }
    case "intersection": {
      return intersection(
        node.types.map((node2) => visitNode(node2, visitor))
      );
    }
    case "union": {
      return union(node.types.map((node2) => visitNode(node2, visitor)));
    }
    case "named": {
      return named(visitNode(node.node, visitor), node.name);
    }
    case "iterator": {
      return iterator(visitNode(node.type, visitor));
    }
    case "custom": {
      return custom(
        node.name,
        node.arguments?.map((node2) => visitNode(node2, visitor))
      );
    }
    default: {
      return node;
    }
  }
}
export {
  visitEachChild,
  visitNode
};
